/* _DirectInternals.c generated by valac 0.32.0.15-114e9, the Vala compiler
 * generated from _DirectInternals.vala, do not modify */

/* Copyright 2016 Software Freedom Conservancy Inc.
 *
 * This software is licensed under the GNU Lesser General Public License
 * (version 2.1 or later).  See the COPYING file in this distribution.
 *
 * Auto-generated file.  Do not modify!
 */

#include <glib.h>
#include <glib-object.h>



extern gint direct__unit_init_count;
gint direct__unit_init_count = 0;

void direct_init_entry (GError** error);
void unit_init_entry (GError** error);
void db_init_entry (GError** error);
void util_init_entry (GError** error);
void photos_init_entry (GError** error);
void slideshow_init_entry (GError** error);
void core_init_entry (GError** error);
void direct_init (GError** error);
void direct_terminate_entry (void);
void direct_terminate (void);
void unit_terminate_entry (void);
void db_terminate_entry (void);
void util_terminate_entry (void);
void photos_terminate_entry (void);
void slideshow_terminate_entry (void);
void core_terminate_entry (void);


void direct_init_entry (GError** error) {
	gint _tmp0_ = 0;
	GError * _inner_error_ = NULL;
#line 14 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	_tmp0_ = direct__unit_init_count;
#line 14 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	direct__unit_init_count = _tmp0_ + 1;
#line 14 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	if (_tmp0_ != 0) {
#line 15 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		return;
#line 50 "_DirectInternals.c"
	}
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	unit_init_entry (&_inner_error_);
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		g_propagate_error (error, _inner_error_);
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		return;
#line 60 "_DirectInternals.c"
	}
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	db_init_entry (&_inner_error_);
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		g_propagate_error (error, _inner_error_);
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		return;
#line 70 "_DirectInternals.c"
	}
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	util_init_entry (&_inner_error_);
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		g_propagate_error (error, _inner_error_);
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		return;
#line 80 "_DirectInternals.c"
	}
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	photos_init_entry (&_inner_error_);
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		g_propagate_error (error, _inner_error_);
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		return;
#line 90 "_DirectInternals.c"
	}
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	slideshow_init_entry (&_inner_error_);
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		g_propagate_error (error, _inner_error_);
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		return;
#line 100 "_DirectInternals.c"
	}
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	core_init_entry (&_inner_error_);
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		g_propagate_error (error, _inner_error_);
#line 17 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		return;
#line 110 "_DirectInternals.c"
	}
#line 19 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	direct_init (&_inner_error_);
#line 19 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	if (G_UNLIKELY (_inner_error_ != NULL)) {
#line 19 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		g_propagate_error (error, _inner_error_);
#line 19 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		return;
#line 120 "_DirectInternals.c"
	}
}


void direct_terminate_entry (void) {
	gboolean _tmp0_ = FALSE;
	gint _tmp1_ = 0;
#line 23 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	_tmp1_ = direct__unit_init_count;
#line 23 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	if (_tmp1_ == 0) {
#line 23 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		_tmp0_ = TRUE;
#line 134 "_DirectInternals.c"
	} else {
		gint _tmp2_ = 0;
		gint _tmp3_ = 0;
#line 23 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		_tmp2_ = direct__unit_init_count;
#line 23 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		direct__unit_init_count = _tmp2_ - 1;
#line 23 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		_tmp3_ = direct__unit_init_count;
#line 23 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		_tmp0_ = _tmp3_ != 0;
#line 146 "_DirectInternals.c"
	}
#line 23 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	if (_tmp0_) {
#line 24 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
		return;
#line 152 "_DirectInternals.c"
	}
#line 26 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	direct_terminate ();
#line 28 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	unit_terminate_entry ();
#line 28 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	db_terminate_entry ();
#line 28 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	util_terminate_entry ();
#line 28 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	photos_terminate_entry ();
#line 28 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	slideshow_terminate_entry ();
#line 28 "/home/jens/Source/shotwell/src/.unitize/_DirectInternals.vala"
	core_terminate_entry ();
#line 168 "_DirectInternals.c"
}



